@@ -19,6 +19,10 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef __FreeBSD__
+#include <sys/types.h>
+#endif
+#include <netinet/in.h>
 
 #include "kodak_generic.h"
 #include "state_machine.h"
@@ -29,7 +33,16 @@
 * Preprocessor Defines
 ==============================================================================*/
 
-#define HIGH_HPBS    8192
+/* NOTE: These are really the data size, not the Host Packet Buffer Size
+ * (HPBS) used in the protocol -- packet framing bytes are omitted.
+ */
+#define HPBS_MIN     0x0200           /* minimum data size */
+#define HPBS_MAX     0x8000           /* maximum data size */
+
+/* Currently getting USB data overruns on larger packets; not yet clear why.
+ * Workaround: keep packet size under 8K.
+ */
+#define HPBS_USB_MAX 0x1FF0
 
 /*==============================================================================
 * Type definitions
@@ -41,15 +54,15 @@ typedef struct
    unsigned short creation_time;
    unsigned short creation_date;
    unsigned char size[4];
-} DIRECTORY_ENTRY_TYPE;
+} DIRECTORY_ENTRY_TYPE;			/* from camera */
 
 typedef struct
 {
-   unsigned char *filename;
+   unsigned char filename [50];
    unsigned long thumbnail_size;
    unsigned long picture_size;
    unsigned char valid;
-} PICTURE_TYPE;
+} PICTURE_TYPE;				/* constructed here */
 
 typedef struct
 {
@@ -63,13 +76,19 @@ typedef struct
    unsigned char adapter_flag;
    unsigned char strobe_status;
    unsigned char padding[256 - 11];
-} STATUS_PACKET_TYPE;
+} STATUS_PACKET_TYPE;			/* from camera */
+
+typedef struct
+{
+   BOOLEAN valid;
+   unsigned char *filename;
+} PICTURE_FILENAME_PACKET_TYPE;
 
 typedef struct
 {
    unsigned short num_entries;
    DIRECTORY_ENTRY_TYPE entries[1];
-} DIRECTORY_PACKET_DATA_TYPE;
+} DIRECTORY_PACKET_DATA_TYPE;		/* from camera */
 
 typedef struct
 {
@@ -114,10 +133,18 @@ static CC_RESPONSE_TYPE kdc240_get_picture_info_read(
    PICTURE_INFO_PACKET_TYPE *, unsigned char *);
 
 static struct Image *kdc240_get_preview(void);
+
 static int kdc240_delete_picture(int);
+static unsigned char *kdc240_delete_picture_tx_filename (
+   DIRECTORY_PACKET_TYPE *);
+
 static int kdc240_take_picture(void);
+static CC_RESPONSE_TYPE kdc240_take_picture_read (
+   PICTURE_FILENAME_PACKET_TYPE *, unsigned char *);
 
 static int kdc240_number_of_pictures(void);
+static void kdc240_get_picture_directory (unsigned char *);
+static void kdc240_add_picture (unsigned char *, unsigned char *);
 static unsigned char *kdc240_number_of_pictures_tx_filename (
    DIRECTORY_PACKET_TYPE *);
 static CC_RESPONSE_TYPE kdc240_number_of_pictures_read (
@@ -133,7 +160,8 @@ static char *kdc240_description(void);
 
 static void kdc240_open_card(void);
 static void kdc240_close_card(void);
-static void kdc240_set_hpbs(void);
+static int kdc240_get_reasonable_hpbs(void);
+static void kdc240_set_hpbs(int);
 
 static unsigned char *kdc240_create_filename(unsigned char *,
    unsigned long, unsigned long);
@@ -143,6 +171,9 @@ static unsigned char *kdc240_create_filename(unsigned char *,
 ==============================================================================*/
 STATE_MACHINE_INSTANCE *machine;
 static PICTURE_TYPE *picture_index = NULL;
+static int number_of_pictures = 0;
+static int hpbs = HPBS_MIN;
+static int num_errors = 0;
 
 /*==============================================================================
 * Constants
@@ -192,6 +223,12 @@ kdc240_post_initialize
 
    /* Fire up the camera */
    kdc240_restart();
+
+   /* maximize USB packet sizes */
+   if (mach->is_usb)
+   {
+      kdc240_set_hpbs(HPBS_USB_MAX);
+   }
 }
 
 /*******************************************************************************
@@ -246,20 +283,35 @@ kdc240_get_picture
    int thumbnail
 )
 {
+   unsigned char *status;
    struct Image *image = NULL;
    PICTURE_PACKET_TYPE buffer;
    CC_STRUCT_TYPE cc_struct = 
    {
       (int)&buffer,
       58,  (void *)kdc240_get_picture_tx_filename,
-      HIGH_HPBS, (void *)kdc240_get_picture_read
+      hpbs, (void *)kdc240_get_picture_read
    };
 
+   /* If we haven't retrieved the index yet, do that first. */
+   if (picture_index == NULL)
+   {
+      kdc240_number_of_pictures();
+   }
+
+   /* Make sure that the picture number is present in the camera. */
+   if (picture > number_of_pictures)
+   {
+      return NULL;
+   }
+
    /* gphoto uses a one-based index, turn it into a zero-based one. */
    picture--;
 
    kdc240_open_card();
 
+   kdc240_set_hpbs(hpbs);
+
    /* Get picture information if we haven't already */
    if (!picture_index[picture].valid)
    {
@@ -277,13 +329,33 @@ kdc240_get_picture
    buffer.filename = kdc240_create_filename(picture_index[picture].filename,
       0, 0);
 
-   kdc240_set_hpbs();
+   status = (unsigned char *)malloc(256);
+   strcpy (status, "Fetching file ");
+   strcat (status, picture_index[picture].filename);
+   update_status(status);
+   free(status);
 
    if (thumbnail)
    {
+      int saved_hpbs = hpbs;
+    
       buffer.size = picture_index[picture].thumbnail_size;
+
+      /* Optimize HPBS for thumbnail retrieval */
+      if (hpbs > buffer.size && buffer.size > HPBS_MIN)
+      {
+         kdc240_set_hpbs (buffer.size);
+         cc_struct.rx_packet_size = buffer.size;
+      }
+
+      /*
+       * NOTE: DC-{50,120} have a different thumbnail command.
+       * NOTE: DC-{200,210} don't support format 2 (JPG) here.
+       */
       kdc240_complex_command(&cc_struct, KODAK_CAMERA_DC240,
          CMD_READ_THUMBNAIL, 2);
+
+      kdc240_set_hpbs (saved_hpbs);
    }
    else
    {
@@ -301,12 +373,24 @@ kdc240_get_picture
       image->image = buffer.rx_buffer;
       strcpy(image->image_type, "jpg");
       image->image_info_size = 0;
+
+      if (num_errors > 0)
+      {
+         num_errors--;
+      }
    }
-   else if (buffer.rx_buffer)
+   else
    {
-      free(buffer.rx_buffer);
+      if (buffer.rx_buffer)
+      {
+         free(buffer.rx_buffer);
+      }
+
+      num_errors++;
    }
 
+   kdc240_set_hpbs(kdc240_get_reasonable_hpbs());
+
    return image;
 }
 
@@ -342,9 +426,9 @@ kdc240_get_picture_read
 
    bytes_to_copy = buffer->size - buffer->rx_bytes;
 
-   if (bytes_to_copy > HIGH_HPBS)
+   if (bytes_to_copy > hpbs)
    {
-      bytes_to_copy = HIGH_HPBS;
+      bytes_to_copy = hpbs;
       retval = CC_RESPONSE_MORE_PACKETS;
    }
    else
@@ -356,7 +440,11 @@ kdc240_get_picture_read
    memcpy(buffer->rx_buffer + buffer->rx_bytes, packet, bytes_to_copy);
    buffer->rx_bytes += bytes_to_copy;
 
+#ifdef READ_DEBUG
    printf ("kdc240_get_picture_read: read %d bytes\n", buffer->rx_bytes);
+#endif
+
+   update_progress((float)buffer->rx_bytes / buffer->size);
 
    return retval;
 }
@@ -376,11 +464,14 @@ kdc240_get_picture_info
       256, (void *)kdc240_get_picture_info_read,
    };
 
+#ifdef PICTURE_INFO_DEBUG
    printf ("kdc240_get_picture_info: filename %s\n", picture->filename);
+#endif
 
    buffer.valid = FALSE;
    buffer.filename = kdc240_create_filename(picture->filename, 0, 0);
    
+   /* NOTE:  DC-{50,120} don't support this command. */
    kdc240_complex_command(&cc_struct, KODAK_CAMERA_DC240, CMD_READ_PIC_INFO);
 
    if (buffer.valid)
@@ -399,8 +490,10 @@ kdc240_get_picture_info
 
       picture->valid = TRUE;
 
+#ifdef PICTURE_INFO_DEBUG
       printf ("        thumbnail size: %ld\n", picture->thumbnail_size);
       printf ("        picture size:   %ld\n", picture->picture_size);
+#endif
    }
 }
 
@@ -440,6 +533,14 @@ kdc240_get_preview
    void
 )
 {
+   struct Image *image;
+   int picture;
+
+   picture = kdc240_take_picture();
+   image = kdc240_get_picture(picture, 0);
+   kdc240_delete_picture(picture);
+
+   return image;
 }
 
 /*******************************************************************************
@@ -453,6 +554,67 @@ kdc240_delete_picture
    int picture
 )
 {
+   DIRECTORY_PACKET_TYPE buffer;
+   CC_STRUCT_TYPE cc_struct = 
+   {
+      (int)&buffer,
+      58,  (void *)kdc240_delete_picture_tx_filename,
+      0,   NULL
+   };
+
+   /* If we haven't retrieved the index yet, do that first. */
+   if (picture_index == NULL)
+   {
+      kdc240_number_of_pictures();
+   }
+
+   /* Make sure that the picture number is present in the camera. */
+   if (picture > number_of_pictures)
+   {
+      return GPHOTO_FAIL;
+   }
+
+   /* gphoto uses a one-based index, turn it into a zero-based one. */
+   picture--;
+
+   kdc240_open_card();
+
+   /* Get picture information if we haven't already */
+   if (!picture_index[picture].valid)
+   {
+      kdc240_get_picture_info(&picture_index[picture]);
+      if (!picture_index[picture].valid)
+      {
+         kdc240_close_card();
+         return GPHOTO_FAIL;
+      }
+   }
+   
+   buffer.valid = FALSE;
+   buffer.rx_bytes = 0;
+   buffer.rx_buffer = NULL;
+   buffer.filename = kdc240_create_filename(picture_index[picture].filename,
+      0, 0);
+
+   kdc240_complex_command(&cc_struct, KODAK_CAMERA_DC240, CMD_DELETE_FILE);
+
+   free(buffer.filename);
+
+   kdc240_close_card();
+
+   /* Re-sync the picture number mappings */
+   kdc240_number_of_pictures();
+
+   return GPHOTO_SUCCESS;
+}
+
+static unsigned char *
+kdc240_delete_picture_tx_filename
+(
+   DIRECTORY_PACKET_TYPE *buffer
+)
+{
+   return buffer->filename;
 }
 
 /*******************************************************************************
@@ -466,6 +628,76 @@ kdc240_take_picture
    void
 )
 {
+   int i;
+
+   PICTURE_FILENAME_PACKET_TYPE buffer = { FALSE, NULL };
+   CC_STRUCT_TYPE cc_struct = 
+   {
+      (int)&buffer,
+      0, NULL,
+      256, (void *)kdc240_take_picture_read,
+   };
+
+   /* Take the picture */
+   kdc240_command(KODAK_CAMERA_DC240, CMD_TAKE_PICTURE);
+
+   /* Read the filename */
+   kdc240_complex_command(&cc_struct, KODAK_CAMERA_DC240, CMD_SEND_LAST_PIC);
+
+   if (buffer.valid == FALSE)
+   {
+      printf ("kdc240_take_picture: unable to determine filename of last picture!\n");
+      return 0;
+   }
+   
+   printf ("kdc240_take_picture: filename %s\n", buffer.filename);
+
+   /* Re-sync the picture number mappings */
+   kdc240_number_of_pictures();
+
+   for (i = 0; i < number_of_pictures; i++)
+   {
+      printf ("kdc240_take_picture: directory entry %d = %s\n", i, picture_index[i].filename);
+      if (strcmp(picture_index[i].filename, buffer.filename) == 0)
+      {
+         free(buffer.filename);
+         return i + 1;
+      }
+   }
+
+   free(buffer.filename);
+   return 0;
+}
+
+static CC_RESPONSE_TYPE
+kdc240_take_picture_read
+(
+   PICTURE_FILENAME_PACKET_TYPE *buffer,
+   unsigned char *packet
+)
+{
+   if (packet != NULL)
+   {
+      buffer->filename = (unsigned char *)malloc(256);
+
+      if (buffer->filename != NULL)
+      {
+         buffer->valid = TRUE;
+         memcpy(buffer->filename, packet, 256);
+      }
+   }
+
+   return CC_RESPONSE_LAST_PACKET;
+}
+
+static int compare_picture_names (
+   const void *left,
+   const void *right
+)
+{
+   const PICTURE_TYPE *l = (const PICTURE_TYPE *) left;
+   const PICTURE_TYPE *r = (const PICTURE_TYPE *) right;
+   return strncmp (l->filename, r->filename, sizeof *l);
 }
 
 /*******************************************************************************
@@ -479,7 +711,6 @@ kdc240_number_of_pictures
    void
 )
 {
-   int retval;
    int i;
 
    DIRECTORY_PACKET_TYPE buffer;
@@ -493,7 +724,7 @@ kdc240_number_of_pictures
    kdc240_open_card();
 
    buffer.valid = FALSE;
-   buffer.filename = kdc240_create_filename("\\DCIM\\100DC240\\*.JPG", 0, 0);
+   buffer.filename = kdc240_create_filename("\\DCIM\\*.*", 0, 0);
    buffer.rx_bytes = 0;
    buffer.rx_buffer = NULL;
 
@@ -501,51 +732,136 @@ kdc240_number_of_pictures
 
    free(buffer.filename);
 
-   kdc240_close_card();
+   /* Free the old picture index (if any) */
+   if (picture_index)
+   {
+      free(picture_index);
+   }
+
+   number_of_pictures = 0;
 
-   retval = 0;
    if (buffer.valid)
    {
-      if (picture_index)
+      for (i = 0; i < buffer.rx_buffer->num_entries; i++)
       {
-         free(picture_index);
-      }
-
-      picture_index = (PICTURE_TYPE *)malloc(
-         sizeof(PICTURE_TYPE) * buffer.rx_buffer->num_entries);
-
-      if (picture_index == NULL)
-      {
-         printf ("kdc240_number_of_pictures: picture_index == NULL\n");
-      }
-      else
-      {
-         retval = buffer.rx_buffer->num_entries;
-         for (i = 0; i < retval; i++)
+         if (buffer.rx_buffer->entries[i].filename[0] != '.')
          {
-             PICTURE_TYPE *t = &picture_index[i];
+            unsigned char *filename = (unsigned char *)malloc(20);
 
-             t->valid = FALSE;
-             t->filename = (unsigned char *)malloc(128);
-             memset(t->filename, 0, 128);
-             strcpy (t->filename, "\\DCIM\\100DC240\\");
-             memcpy (t->filename + strlen(t->filename),
-                buffer.rx_buffer->entries[i].filename, 8);
-             t->filename[strlen(t->filename)] = '.';
-             memcpy (t->filename + strlen(t->filename),
-                buffer.rx_buffer->entries[i].filename + 8, 3);
+            memset(filename, 0, 20);
+            strcpy(filename, "\\DCIM\\");
+            memcpy(filename + 6, buffer.rx_buffer->entries[i].filename, 11);
+            *strchr(filename, ' ') = '\0';
+            strcat(filename, "\\");
 
-             printf ("    Entry %d: %s\n", i, t->filename);
+            kdc240_get_picture_directory(filename);
+
+            free(filename);
          }
       }
+
+      /* Sort the pictures in the list */
+      qsort (picture_index, number_of_pictures, sizeof (picture_index[0]),
+         compare_picture_names);
+   }
+
+   kdc240_close_card();
+
+   if (buffer.rx_buffer)
+   {
+      free(buffer.rx_buffer);
+   }
+
+   return number_of_pictures;
+}
+
+static void
+kdc240_get_picture_directory
+(
+   unsigned char *directory
+)
+{
+   unsigned char *filename;
+   int i;
+
+   DIRECTORY_PACKET_TYPE buffer;
+   CC_STRUCT_TYPE cc_struct = 
+   {
+      (int)&buffer,
+      58,  (void *)kdc240_number_of_pictures_tx_filename,
+      256, (void *)kdc240_number_of_pictures_read
+   };
+
+   filename = (unsigned char *)malloc(strlen(directory) + 10);
+   strcpy(filename, directory);
+   strcat(filename, "*.JPG");
+
+#ifdef DIRECTORY_DEBUG
+   printf ("  searching: %s\n", filename);
+#endif
+
+   buffer.valid = FALSE;
+   buffer.filename = kdc240_create_filename(filename, 0, 0);
+   buffer.rx_bytes = 0;
+   buffer.rx_buffer = NULL;
+
+   kdc240_complex_command(&cc_struct, KODAK_CAMERA_DC240, CMD_READ_DIRECTORY);
+
+   free(filename);
+   free(buffer.filename);
+
+   if (buffer.valid)
+   {
+      for (i = 0; i < buffer.rx_buffer->num_entries; i++)
+      {
+         kdc240_add_picture(directory, buffer.rx_buffer->entries[i].filename);
+      }
    }
 
    if (buffer.rx_buffer)
    {
       free(buffer.rx_buffer);
    }
+}
 
-   return retval;
+static void
+kdc240_add_picture
+(
+   unsigned char *directory,
+   unsigned char *filename
+)
+{
+   PICTURE_TYPE *t;
+
+   /* If this is the first picture, initialize the pointer */
+   if (number_of_pictures == 0)
+   {
+      picture_index = NULL;
+   }
+
+   /* Increment the total number of pictures */
+   number_of_pictures++;
+
+   /* Allocate space for the new picture */
+   picture_index = (PICTURE_TYPE *)realloc(picture_index,
+      sizeof(PICTURE_TYPE) * number_of_pictures);
+
+   /* Create a temporary pointer to the new picture */
+   t = &picture_index[number_of_pictures - 1];
+
+   /* Indicate that we haven't gotten information on this picture yet */
+   t->valid = FALSE;
+
+   /* Fill in the filename */
+   memset(t->filename, 0, sizeof(t->filename));
+   strcpy (t->filename, directory);
+   memcpy (t->filename + strlen(t->filename), filename, 8);
+   t->filename[strlen(t->filename)] = '.';
+   memcpy (t->filename + strlen(t->filename), filename + 8, 3);
+
+#ifdef DIRECTORY_DEBUG
+   printf ("adding %s\n", t->filename);
+#endif
 }
 
 static unsigned char *
@@ -657,21 +973,33 @@ kdc240_summary
       strcat (summary_buf, "Camera is a Kodak Digital Science ");
       switch (buffer.camera_type)
       {
+         case 1:
+            strcat (summary_buf, "DC50\n");
+            break;
+
+         case 2:
+            strcat (summary_buf, "DC120\n");
+            break;
+
          case 3:
-             strcat (summary_buf, "DC200\n");
-             break;
+            strcat (summary_buf, "DC200\n");
+            break;
 
          case 4:
-             strcat (summary_buf, "DC210\n");
-             break;
+            strcat (summary_buf, "DC210\n");
+            break;
 
          case 5:
-             strcat (summary_buf, "DC240\n");
-             break;
+            strcat (summary_buf, "DC240\n");
+            break;
+
+         case 6:
+            strcat (summary_buf, "DC280\n");
+            break;
 
          default:
-             strcat (summary_buf, "(unknown)\n");
-             break;
+            strcat (summary_buf, "(unknown)\n");
+            break;
       }
 
       strcat (summary_buf,"Firmware version: ");
@@ -681,37 +1009,47 @@ kdc240_summary
       strcat (summary_buf,"Battery status: ");
       switch (buffer.battery_status)
       {
-          case 0:
-              strcat (summary_buf, "OK\n");
-              break;
+         case 0:
+            strcat (summary_buf, "OK\n");
+            break;
 
-          case 1:
-              strcat (summary_buf, "Weak\n");
-              break;
+         case 1:
+            strcat (summary_buf, "Weak\n");
+            break;
 
-          case 2:
-              strcat (summary_buf, "Empty\n");
-              break;
+         case 2:
+            strcat (summary_buf, "Empty\n");
+            break;
 
-          default:
-              strcat (summary_buf, "(error)\n");
-              break;
+         default:
+            strcat (summary_buf, "(error)\n");
+            break;
       }
 
       strcat (summary_buf,"AC Adapter status: ");
       switch (buffer.adapter_flag)
       {
-          case 0:
-              strcat (summary_buf, "not connected\n");
-              break;
+         case 0:
+            strcat (summary_buf, "not connected\n");
+            break;
 
-          case 1:
-              strcat (summary_buf, "connected\n");
-              break;
+         case 1:
+            strcat (summary_buf, "connected\n");
+            break;
 
-          default:
-              strcat (summary_buf, "(error)\n");
-              break;
+         default:
+            strcat (summary_buf, "(error)\n");
+            break;
+      }
+
+      strcat (summary_buf,"Connection: ");
+      if (machine->is_usb)
+      {
+         strcat (summary_buf,"USB\n");
+      }
+      else
+      {
+         strcat (summary_buf,"Serial\n");
       }
 
       return summary_buf;
@@ -770,13 +1108,72 @@ kdc240_close_card
    kdc240_command(KODAK_CAMERA_DC240, CMD_CLOSE_CARD);
 }
 
-static void
-kdc240_set_hpbs
+static int
+kdc240_get_reasonable_hpbs
 (
    void
 )
 {
-   kdc240_command(KODAK_CAMERA_DC240, CMD_SET_HPBS, HIGH_HPBS + 2);
+   int new_hpbs = hpbs;
+
+   /* HPBS for USB is fixed */
+   if (machine->is_usb)
+   {
+      new_hpbs = HPBS_USB_MAX;
+   }
+
+   /* For serial, we attempt to base the HPBS on the error rate */
+   else
+   {
+      if (hpbs == HPBS_MIN && num_errors > 0)
+      {
+         /* Do nothing, we can't lower HPBS any more */
+      }
+      else if (num_errors > 0)
+      {
+         /* Lower HPBS */
+         new_hpbs = (HPBS_MIN + hpbs) >> 1;
+      }
+      else
+      {
+         /* Raise HPBS */
+         new_hpbs = (HPBS_MAX + hpbs) >> 1;
+      }
+
+      /* Error check the resulting HPBS before sending to camera */
+      if (new_hpbs > HPBS_MAX)
+      {
+         new_hpbs = HPBS_MAX;
+      }
+      else if (new_hpbs < HPBS_MIN)
+      {
+         new_hpbs = HPBS_MIN;
+      }
+   }
+
+   return new_hpbs;
+}
+
+static void
+kdc240_set_hpbs
+(
+   int new_hpbs
+)
+{
+   /* If the HPBS hasn't changed, don't bother writing to the camera. */
+   if (new_hpbs == hpbs)
+   {
+      return;
+   }
+
+   /* NOTE: DC-{50,120,200,210} have fixed HPBS */
+   kdc240_command(KODAK_CAMERA_DC240, CMD_SET_HPBS, new_hpbs + 2);
+
+#ifdef HPBS_DEBUG
+   printf ("kdc240_set_hpbs: hpbs set to %d\n", new_hpbs + 2);
+#endif
+
+   hpbs = new_hpbs;
 }
 
 static unsigned char *
